import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import { DataloadSectionType, DataloadState, FluidType } from 'enums'
import { Dataload } from 'models'
import React from 'react'
import { NavLink } from 'react-router-dom'
import ConverterService from 'services/converter.service'
import { getFluidIcon } from 'utils/picto'
import { formatNumberValues, getFluidName } from 'utils/utils'

interface DataloadSectionDetailProps {
  dataload: Dataload
  fluidType: FluidType
  dataloadSectionType: DataloadSectionType
  focusable?: boolean
}

const DataloadSectionDetail = ({
  dataload,
  fluidType,
  dataloadSectionType,
  focusable,
}: DataloadSectionDetailProps) => {
  const { t } = useI18n()
  const converterService = new ConverterService()

  const isMulti = fluidType === FluidType.MULTIFLUID
  const isCompare = dataloadSectionType === DataloadSectionType.NO_COMPARE

  const fluidName = getFluidName(fluidType)

  if (!isMulti) {
    return (
      <div
        className={classNames('dataloadvisualizer-euro text-16-normal', {
          [fluidName]: dataloadSectionType !== DataloadSectionType.LEFT,
          [`${fluidName}-compare`]:
            dataloadSectionType === DataloadSectionType.LEFT,
        })}
      >
        {`${formatNumberValues(
          converterService.LoadToEuro(
            dataload.value,
            fluidType,
            dataload.price ? dataload.price : null
          )
        )} €`}
      </div>
    )
  }

  if (isMulti && !isCompare) {
    return <div className="dataloadvisualizer-euro text-16-normal" />
  }

  if (isMulti && isCompare && dataload.valueDetail) {
    return (
      <div role="group" className="dataloadvisualizer-euro text-16-normal">
        {dataload.valueDetail.map((value, index) => {
          const isValid = value.state === DataloadState.VALID
          const isUpcoming = value.state === DataloadState.UPCOMING
          const isComing = value.state === DataloadState.COMING
          const isEmpty = value.state === DataloadState.EMPTY
          const isHole = value.state === DataloadState.HOLE
          const isMissing = value.state === DataloadState.MISSING
          const fluidName = getFluidName(index)
          return (
            <NavLink
              key={FluidType[index]}
              aria-label={t(
                `consumption_visualizer.cost_per_fluid.${fluidName}`
              )}
              to={`/consumption/${fluidName}`}
              className="dataloadvisualizer-euro-link"
              tabIndex={focusable ? 0 : -1}
            >
              <div
                className={classNames('dataloadvisualizer-euro-fluid', {
                  [fluidName]:
                    isValid || isUpcoming || isComing || isEmpty || isHole,
                  error: isMissing,
                })}
              >
                <StyledIcon icon={getFluidIcon(index)} size={22} />
                <span>
                  {isValid && `${formatNumberValues(value.value)} €`}
                  {(isUpcoming || isComing) &&
                    t('consumption_visualizer.data_to_come')}
                  {(isEmpty || isHole) &&
                    t('consumption_visualizer.data_empty')}
                  {isMissing && t('consumption_visualizer.aie')}
                </span>
              </div>
            </NavLink>
          )
        })}
      </div>
    )
  }

  return (
    <div className="dataloadvisualizer-euro text-16-normal">
      <NavLink
        to={`/consumption/${fluidName}`}
        className="dataloadvisualizer-euro-link"
        tabIndex={focusable ? 0 : -1}
      >
        <div className={`dataloadvisualizer-euro-fluid ${fluidName}`}>
          <StyledIcon icon={getFluidIcon(fluidType)} size={22} />
          <span>{`${formatNumberValues(
            converterService.LoadToEuro(
              dataload.value,
              fluidType,
              dataload.price ? dataload.price : null
            )
          )} €`}</span>
        </div>
      </NavLink>
    </div>
  )
}

export default DataloadSectionDetail
